# ircslice.cl -- To slice the IRC 3D images into the 2D ones.
# Created: Yoshifusa Ita, 02-Jul-05
# 05-Jul-05 - Add cleaning (Y. Ita)
# Updated: Yoshifusa Ita, 25-Aug-06, Add header keyword - GAIN and SUBID
# Updated: Yoshifusa Ita, 05-Nov-06, Add header keyword - C(R)PIX1 and C(R)PIX2
# Updated: Takehiko Wada  07-Jan-03, enable to update DETECTOR
#                                    KEYWORD (for MIR)
# Updated: Takehiko Wada  2007/01/03 
#           update HEADER (replace non-alphabet, decimal to "_")
#           $B$&$C$H$&$7$$$N$G!"(Bslice$B8e$N(Bheader$B$K$O!"(BOBJECT$BL>$N1Q?t;z0J(B
#          $B30$NJ8;z$rA4It(B "_"$B$KF~$l49$($^$9!#(B
# Updated: Yoshifusa Ita 2008/01/17
#          imslice$B$r;H$o$J$$$h$&$K$7$F!"%9%i%$%9$7!"(BMIRL$B$N(BWCS$B7O%X%C%@$r(B
#          $B@5$7$/F~$lD>$9$h$&$K$7$?!#$"$H!"ITI,MW$G$R$g$C$H$7$?$i0-5:$9$k$h$&$J(B
#          wcs$B$N%X%C%@$r>C$9$h$&$K$7$?!#(B

procedure ircslice (ircconst, verbose)
string  mode="al"

string  ircconst     {"constants.database", prompt="IRC constants database file name"}
bool    verbose = no {prompt="Print verbose progress messages?\n"}
struct *list0

begin

 string irc_const, libpath
 string fname, finput, foutput, sname, lname
 string tmp1, tmp2, text
 string pipever
 int 	naxis, naxis1, naxis2, naxis3
 bool   verb
 real   epadu_nir, epadu_mirs, epadu_mirl
 string object_name	
 real crval1l, crval2l, crpix1l, crpix2l, cd11l, cd12l, cd21l, cd22l

 print ("### IRCSLICE ###\n")

# get querty parameters
 verb = verbose
 irc_const = ircconst


# edit header
 libpath = osfn ("irclib$")
 irc_const = libpath//irc_const
 # pipeline version
 printf("! grep pipever        %s\n", irc_const) | cl | scan(text, pipever)
 # A2D unit (Gain) [electrons/ADU]
 printf("! grep epadu_nir      %s\n", irc_const) | cl | scan(text, epadu_nir)
 printf("! grep epadu_mirs     %s\n", irc_const) | cl | scan(text, epadu_mirs)
 printf("! grep epadu_mirl     %s\n", irc_const) | cl | scan(text, epadu_mirl)


# make input file list
 if( access("filename.list") ){ delete("filename.list", ver-) }
 print ("Making the input file list...\n")
 ! ls -1 ../rawdata | egrep -e F\*.fits > filename.list


# check the dimension of the fits file
 print ("Checking the dimension of the input files...\n")
 if( access("slice_tmp0") ){ delete("slice_tmp0", ver-) }
 list0 = "filename.list"
 while( fscan(list0, fname) !=EOF){
	fname="../rawdata/"//fname
 	hselect(fname, fields="$I,NAXIS,NAXIS1,NAXIS2,NAXIS3,OBJECT", expr="yes", >> "slice_tmp0")
 }


# slice and imcopy
 print ("Slicing images...\n")
 print ("\t #This may take a while...\n")
# output.list$B$N%U%)!<%^%C%H$O!"(B"FXXXXXXXXX" naxis naxis1 naxis2 naxis3 object_name
# $B$?$@$7(Bobject_name$B$O!"$"$H$G6uGr6h@Z$j$K$h$k<h$j07$$$r2DG=$K$9$k$?$a!"(B
# $B1Q?t;z0J30$r(B"_"$B$KJQ49$7$F$*$/(B
! cat slice_tmp0 | perl -e 'while(<>){ chomp; split(/\t/);$_[0]=~s/.*F([0-9]+)_.*/F$1/;$_[5]=~s/\"//g;$_[5]=~s/\W/_/g;printf("%s %d %d %d %d %s\n",$_[0],$_[1],$_[2],$_[3],$_[4],$_[5]) }' > output.list
 list0 = "output.list"
 while( fscan(list0, fname, naxis, naxis1, naxis2, naxis3, object_name) !=EOF ){
	if(naxis1==412){ # NIR
		finput  = "../rawdata/"//fname//"_N.fits"
		foutput = fname//"_N"
 		imcopy (finput, foutput, verbose=verb)
		# OBJECT$BL>$r(Bupdate$B$7$F$*$/(B
		hedit (foutput, "OBJECT", object_name, add=yes, addonly=no, delete=no, verify=no, show=no, update=yes)
		# Pipeline version
		hedit (foutput, "PIPEVER", "v"//pipever, add=no, addonly=yes, delete=no, verify=no, show=no, update=yes)
		# Detector
		hedit (foutput, "DETECTOR", "NIR",  add=yes, addonly=no, delete=no, verify=no, show=no, update=yes)
		# delete wcsdim
		hedit (foutput, "WCSDIM", "", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		# GAIN [e/adu]
		ccdhedit (foutput, paramete="GAIN", value=epadu_nir, type="real")
		# CPIX1 & CPIX2
		ccdhedit (foutput, paramete="CPIX1", value=197.0, type="real")
		ccdhedit (foutput, paramete="CPIX2", value=191.0, type="real")
		# $B%9%i%$%9$9$k!#(B
		finput = foutput
		imcopy (finput//"[*,*,1]", finput//"001.fits", verbose=verb)
		imcopy (finput//"[*,*,2]", finput//"002.fits", verbose=verb)
		# EXPID [1:short 2:long]
		ccdhedit (foutput//"001.fits", paramete="EXPID", value=1, type="integer")
		ccdhedit (foutput//"002.fits", paramete="EXPID", value=2, type="integer")
		# 080117 WCS$B4XO"(B
		# $BDL>o$N(B2$B<!852hA|$K$J$C$?$N$G!"ITI,MW$JJ*$r$r>C$9(B
		if( access("sliced.list") ){ delete("sliced.list", ver-) }
		printf("! ls -1 %s00?.fits > sliced.list\n", foutput) | cl
		hedit ("@sliced.list", "CD3_3",    ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CTYPE3",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CRPIX3",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "LTV3",     ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "LTM3_3",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "WAXMAP01", ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "WAXMAP01", ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "WCSDIM", "2",  add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		if( access("sliced.list") ){ delete("sliced.list", ver-) }
                imdel(foutput, ver-)
	}else if(naxis1==256){ # MIRS & MIRL
		finput  = "../rawdata/"//fname//"_M.fits"

		# MIRS
		tmp1 = finput//"[1:256,1:256,*]"
		sname = fname//"_S"
 		imcopy (tmp1, sname, verbose=verb)
		foutput = sname
		hedit (foutput, "OBJECT", object_name, add=yes, addonly=no, delete=no, verify=no, show=no, update=yes)
		hedit (foutput, "DETECTOR", "MIRS", add=yes, addonly=no, delete=no, verify=no, show=no, update=yes)
		hedit (foutput, "WCSDIM", "", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit (foutput, "PIPEVER", "v"//pipever, add=no, addonly=yes, delete=no, verify=no, show=no, update=yes)
		ccdhedit (foutput, paramete="GAIN", value=epadu_mirs, type="real")
		ccdhedit (foutput, paramete="CPIX1", value=150.0, type="real")
		ccdhedit (foutput, paramete="CPIX2", value=118.0, type="real")
		imcopy (sname//"[*,*,1]", sname//"001.fits", verbose=verb)
		imcopy (sname//"[*,*,2]", sname//"002.fits", verbose=verb)
		imcopy (sname//"[*,*,3]", sname//"003.fits", verbose=verb)
		imcopy (sname//"[*,*,4]", sname//"004.fits", verbose=verb)
		ccdhedit (foutput//"001.fits", paramete="EXPID", value=1, type="integer")
		ccdhedit (foutput//"002.fits", paramete="EXPID", value=2, type="integer")
		ccdhedit (foutput//"003.fits", paramete="EXPID", value=3, type="integer")
		ccdhedit (foutput//"004.fits", paramete="EXPID", value=4, type="integer")
		# 080117 WCS$B4XO"(B
		# $BDL>o$N(B2$B<!852hA|$K$J$C$?$N$G!"ITI,MW$JJ*$r$r>C$9(B
		if( access("sliced.list") ){ delete("sliced.list", ver-) }
		printf("! ls -1 %s00?.fits > sliced.list\n", foutput) | cl
		hedit ("@sliced.list", "CD3_3",    ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CD3_3S",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CD3_3L",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CTYPE3",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CRPIX3",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "LTV3",     ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "LTM3_3",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "WAXMAP01", ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "WCSDIM", "2",  add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		if( access("sliced.list") ){ delete("sliced.list", ver-) }
		imdel(sname, ver-)

		# MIRL
		tmp2 = finput//"[1:256,257:512,*]"
		lname = fname//"_L"
		imcopy (tmp2, lname, verbose=verb)
		foutput = lname
		hedit (foutput, "OBJECT", object_name, add=yes, addonly=no, delete=no, verify=no, show=no, update=yes)
		hedit (foutput, "DETECTOR", "MIRL", add=yes, addonly=no, delete=no, verify=no, show=no, update=yes)
		hedit (foutput, "WCSDIM", "", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit (foutput, "PIPEVER", "v"//pipever, add=no, addonly=yes, delete=no, verify=no, show=no, update=yes)
		ccdhedit (foutput, paramete="GAIN", value=epadu_mirl, type="real")
		ccdhedit (foutput, paramete="CPIX1", value=121.0, type="real")
		ccdhedit (foutput, paramete="CPIX2", value=121.0, type="real")
		imcopy (lname//"[*,*,1]", lname//"001.fits", verbose=verb)
		imcopy (lname//"[*,*,2]", lname//"002.fits", verbose=verb)
		imcopy (lname//"[*,*,3]", lname//"003.fits", verbose=verb)
		imcopy (lname//"[*,*,4]", lname//"004.fits", verbose=verb)
		ccdhedit (foutput//"001.fits", paramete="EXPID", value=1, type="integer")
		ccdhedit (foutput//"002.fits", paramete="EXPID", value=2, type="integer")
		ccdhedit (foutput//"003.fits", paramete="EXPID", value=3, type="integer")
		ccdhedit (foutput//"004.fits", paramete="EXPID", value=4, type="integer")
		# 080117 WCS$B4XO"(B
		if( access("sliced.list") ){ delete("sliced.list", ver-) }
		printf("! ls -1 %s00?.fits > sliced.list\n", foutput) | cl
		# WCS$B$,(BMIRS$B$N$b$N$,D%$i$l$F$$$k$N$G!"(BMIRL$B$NJ*$KD%$jBX$($k$?$a$N=`Hw(B
		hselect(foutput, fields="CRVAL1L", expr="yes") | scan(crval1l)
		hselect(foutput, fields="CRVAL2L", expr="yes") | scan(crval2l)
		hselect(foutput, fields="CRPIX1L", expr="yes") | scan(crpix1l)
		hselect(foutput, fields="CRPIX2L", expr="yes") | scan(crpix2l)
		crpix2l = crpix2l - 256
		hselect(foutput, fields="CD1_1L",  expr="yes") | scan(cd11l)
		hselect(foutput, fields="CD1_2L",  expr="yes") | scan(cd12l)
		hselect(foutput, fields="CD2_1L",  expr="yes") | scan(cd21l)
		hselect(foutput, fields="CD2_2L",  expr="yes") | scan(cd22l)
		# $BDL>o$N(B2$B<!852hA|$K$J$C$?$N$G!"ITI,MW$JJ*$r$r>C$9(B
		hedit ("@sliced.list", "CD3_3",    ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CD3_3S",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CD3_3L",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CTYPE3",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CRPIX3",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "LTV2",     ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "LTV3",     ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "LTM3_3",   ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "WAXMAP01", ".", add=no, addonly=no, delete=yes, verify=no, show=no, update=yes)
		# WCS$B$r(BMIRL$B$NJ*$KD%$jBX$($k(B
		hedit ("@sliced.list", "CRVAL1", crval1l, add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CRVAL2", crval2l, add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CRPIX1", crpix1l, add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CRPIX2", crpix2l, add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CD1_1", cd11l, add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CD1_2", cd12l, add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CD2_1", cd21l, add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "CD2_2", cd22l, add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		hedit ("@sliced.list", "WCSDIM", "2",  add=yes, addonly=yes, delete=no, verify=no, show=no, update=yes)
		if( access("sliced.list") ){ delete("sliced.list", ver-) }
		imdel(lname, ver-)
	}
 }

# cleaning
 if( access("filename.list") ){ delete("filename.list", ver-) }
 if( access("slice_tmp0")    ){ delete("slice_tmp0", ver-)    }
 if( access("output.list")   ){ delete("output.list", ver-)   }

 print ("### IRCSLICE finished!! ###\n")

end

